import vaultAgent from "../../utils/vault-agent";

export abstract class Vault<T> {
  data?: T;
  abstract get vaultPath(): string;
  async load(reload = false): Promise<T> {
    if (reload || !this.data) {
      await this.pull();
    }
    return this.data as T;
  }
  async pull(): Promise<void> {
    this.data = await vaultAgent.read<T>(this.vaultPath);
  }
  async push(): Promise<void> {
    await vaultAgent.write(this.vaultPath, this.data);
  }
}

export abstract class OneWaySyncVault<T> extends Vault<T> {
  abstract pullAlt(): Promise<void>;
  async sync(): Promise<void> {
    await this.pullAlt();
    await this.push();
  }
}

export abstract class SyncVault<T> extends OneWaySyncVault<T> {
  abstract pushAlt(): Promise<void>;
  async reverseSync(): Promise<void> {
    await this.pull();
    await this.pushAlt();
  }
}
