import os from "os";

import OSS from "ali-oss";

import { opGet } from "../../utils/1password";

import { OneWaySyncVault } from "./abstracts";

interface AliyunRam {
  Ali_Key: string;
  Ali_Secret: string;
}

export default class AliyunRamVault extends OneWaySyncVault<AliyunRam> {
  constructor(
    public accountName: string,
    public title: string = os.hostname()
  ) {
    super();
  }
  get vaultPath(): string {
    return `geektr.co/aliyun/${this.accountName}/ram/${this.title}`;
  }
  async pullAlt(): Promise<void> {
    const opItem = await opGet({
      vault: "Cloud",
      item: `aliyun/${this.accountName}:secret:ram-users`,
    });
    this.data = opItem.sections[this.title] as unknown as AliyunRam;
  }
  async getProgramRam(): Promise<OSS.Options> {
    const data = await this.load();
    return {
      accessKeyId: data.Ali_Key,
      accessKeySecret: data.Ali_Secret,
    };
  }
  static async pullAltByAccount(
    accountName: string
  ): Promise<AliyunRamVault[]> {
    const { sections } = await opGet({
      vault: "Cloud",
      item: `aliyun/${accountName}:secret:ram-users`,
    });
    return Object.keys(sections).map((title) => {
      const section = sections[title];
      const vault = new AliyunRamVault(accountName, title);
      vault.data = section as unknown as AliyunRam;
      return vault;
    });
  }
}

export const MyAliyunRamsVault = new AliyunRamVault("geektr");
