import { Command } from "commander";

import { CertificateVault } from "../core/vaults/certificate";

interface RenewOptions {
  primaryDomain: string;
  alternativeDomains?: string[];
}

async function renew(
  { primaryDomain, alternativeDomains }: RenewOptions,
  keepAltStore?: boolean
) {
  await new CertificateVault(primaryDomain, alternativeDomains).renew(
    keepAltStore
  );
}

async function renewAll(keepAltStore?: boolean) {
  const vaults = await CertificateVault.pullAll();
  for (const vault of vaults) {
    if (vault.needsRenew) {
      await vault.renew(keepAltStore);
    }
  }
}

export default function (program: Command): void {
  const acmeSh = program
    .command("acme.sh")
    .description("manager certificates created by acme.");

  acmeSh
    .command("renew-all")
    .description("renew all certs")
    .option("--keep", "keep local file in acme.sh dir")
    .action(async ({ keep }) => await renewAll(keep));

  acmeSh
    .command("renew <primaryDomain> [alternativeDomains...]")
    .description("renew or issue cert")
    .option("--keep", "keep local file in acme.sh dir")
    .action(
      async (primaryDomain, alternativeDomains, { keep }) =>
        await renew({ primaryDomain, alternativeDomains }, keep)
    );
}
