import { promises as fs } from "fs";
import path from "path";

import axios, { AxiosResponse } from "axios";
import { Command } from "commander";
import _pretty from "json-stringify-pretty-compact";

import { YumemiCoolqVault } from "../core/vaults/coolq";
import { readMultiLine } from "../utils/read-stdin";

const pretty = (data: unknown): string =>
  _pretty(data, { maxLength: 80, indent: 2 });

const prettyLog = (data: unknown): void => console.log(pretty(data));

const initCtx = async () => {
  const { baseURL, access_token, admin } = await YumemiCoolqVault.load();
  const fetch = async (
    url: string,
    params?: Record<string, unknown>
  ): Promise<AxiosResponse> =>
    (await axios.get(url, { baseURL, params: { ...params, access_token } }))
      .data.data;
  return { fetch, admin };
};

export default function (program: Command): void {
  const qq = program.command("qq").description("qq");

  qq.command("info")
    .description("coolq account info")
    .action(async () => {
      const { fetch } = await initCtx();
      prettyLog(await fetch("/get_login_info"));
    });

  qq.command("groups")
    .description("coolq groups list")
    .action(async () => {
      const { fetch } = await initCtx();
      prettyLog(await fetch("/get_group_list"));
    });

  qq.command("send [message]")
    .description("coolq send message to admin")
    .action(async (message: string | undefined) => {
      if (!message) {
        message = (await readMultiLine("message> ")).join("\n");
      } else if (message.startsWith("@")) {
        const file = message.slice(1);
        message = await fs.readFile(path.resolve(file), "utf8");
      }
      const { fetch, admin } = await initCtx();
      await fetch("/send_private_msg", { user_id: admin, message });
      console.warn("Sent !");
    });
}
