import url from "url";

import OSS from "ali-oss";
import axios, { AxiosRequestConfig } from "axios";
import { Command } from "commander";

import { MyAliyunRamsVault } from "../core/vaults/aliyun-ram";

const region = "oss-cn-shanghai";
const bucket = "co-geektr-portal";
const portalUrl = "https://portal.geektr.co";

export default function (program: Command): void {
  program
    .command("portal <resource url>")
    .description("transfer file to portal.geektr.co")
    .option("--proxy [string]", "proxy ")
    .action(async (resourceUrl, { proxy: proxyUrl }) => {
      const client = new OSS({
        ...(await MyAliyunRamsVault.getProgramRam()),
        region,
        bucket,
      });

      const { hostname, pathname } = url.parse(resourceUrl);
      const proxy = proxyUrl && url.parse(proxyUrl);

      const key = `${hostname}${pathname}`;

      console.warn("fetch resource...");
      const options: AxiosRequestConfig = {
        responseType: "stream",
      };

      if (proxy && proxy.hostname && proxy.port) {
        options.proxy = {
          host: proxy.hostname,
          port: Number(proxy.port),
        };
      }

      const resp = await axios.get(resourceUrl, options);

      console.warn("stream uploading...");
      await client.putStream(key, resp.data);

      console.warn("done:");
      console.log(`${portalUrl}/${key}`);
    });
}
