import { Command } from "commander";

import AliyunRamVault from "../../core/vaults/aliyun-ram";
import { YumemiCoolqVault } from "../../core/vaults/coolq";
import { MyDnspodVault } from "../../core/vaults/dnspod";
import { MyGandiVault } from "../../core/vaults/gandi";
import { appendProfile } from "../init-shell";

export default function (program: Command): void {
  appendProfile('vault-init() { eval "$(yumemi vault init ${1})" }');

  const vault = program
    .command("vault")
    .description("manager secrets in vault");

  vault
    .option("--update [boolean]", "update vault secrets by alternative source")
    .action(async ({ update }) => {
      if (update) {
        await Promise.all<unknown>([
          MyGandiVault.sync(),
          MyDnspodVault.sync(),
          YumemiCoolqVault.sync(),
          AliyunRamVault.pullAltByAccount("geektr").then((arvs) =>
            Promise.all(arvs.map((i) => i.sync()))
          ),
        ]);
      }
    });

  const init = vault
    .command("init")
    .description("init some environment for some service");

  init.command("govc").action(() => require("./init/govc"));
  init.command("aliyun").action(() => require("./init/aliyun"));
  init.command("dnspod").action(() => require("./init/dnspod"));
}
