import inquirer from "inquirer";

import vault from "../../../utils/vault-agent";

import { secretExports } from "./utils";

const prompt = inquirer.createPromptModule({ output: process.stderr });

const vaultBasePath = "geektr.co/govc";
const EmptyTag = { name: "[Empty]", value: "" };

async function init() {
  const hostList = await vault.list(vaultBasePath);
  const { host } = await prompt({
    type: "list",
    name: "host",
    message: "Select Host:",
    choices: hostList,
  });
  const { datastores, networks, url } = await vault.read(
    `${vaultBasePath}/${host}`
  );
  const { datastore, network } = await prompt([
    {
      type: "list",
      name: "datastore",
      message: "Select Datastore:",
      choices: [EmptyTag, ...datastores.split(",")],
    },
    {
      type: "list",
      name: "network",
      message: "Select Network:",
      choices: [EmptyTag, ...networks.split(",")],
    },
  ]);

  secretExports({
    GOVC_URL: url,
    GOVC_DATASTORE: datastore,
    GOVC_NETWORK: network,
  });
}

init();
