interface AssemblyLineOptions<Artifact> {
  artifacts?: Artifact[];
  workers?: number;
}

export default abstract class AssemblyLine<Artifact, Product = void> {
  private artifacts: Artifact[] = [];
  private products: Product[] = [];
  private totalWorkers = 8;
  private workingWorkers = 0;
  private resolve?: (products: Product[]) => void;
  get restingWorkers(): number {
    return this.totalWorkers - this.workingWorkers;
  }
  constructor({ artifacts, workers }: AssemblyLineOptions<Artifact>) {
    if (artifacts) this.artifacts = artifacts;
    if (workers) this.totalWorkers = workers;
  }
  push(artifacts: Artifact | Artifact[]): void {
    artifacts = Array.isArray(artifacts) ? artifacts : [artifacts];
    for (const artifact of artifacts) this.artifacts.push(artifact);
  }
  pop(): Artifact | undefined {
    return this.artifacts.pop();
  }
  abstract work(artifact: Artifact): Promise<Product>;
  private async _run(): Promise<void> {
    if (!this.workingWorkers && this.artifacts.length === 0) {
      if (!this.resolve) throw Error();
      this.resolve(this.products);
      return;
    }

    if (!this.restingWorkers) return;

    const artifact = this.artifacts.pop();
    if (!artifact) return;

    this.workingWorkers += 1;

    if (this.restingWorkers) this._run();

    const product = await this.work(artifact);

    this.products.push(product);

    this.workingWorkers -= 1;
    this._run();
  }
  run(): Promise<Product[]> {
    return new Promise((resolve) => {
      this.resolve = resolve;
      this._run();
    });
  }
}
