// eslint-disable-next-line @typescript-eslint/no-explicit-any
const PromiseCache = new WeakMap<any, Map<string | symbol, Promise<unknown>>>();

function getCache(obj: unknown) {
  const cache = PromiseCache.get(obj);
  if (cache) return cache;
  const newMap = new Map();
  PromiseCache.set(obj, newMap);
  return newMap;
}

interface DebouncePromiseOptions {
  debounceLevel?: "instance" | "class";
}

function debouncePromise(opts: DebouncePromiseOptions = {}) {
  return function (
    target: unknown,
    propertyKey: string,
    descriptor: TypedPropertyDescriptor<() => Promise<void>>
  ): TypedPropertyDescriptor<() => Promise<void>> {
    if (!descriptor.value) {
      throw `'${propertyKey}' has no value`;
    }
    const originFunction = descriptor.value;

    descriptor.value = function (...args) {
      const cacheKey = opts.debounceLevel === "class" ? target : this;
      const cache = getCache(cacheKey);

      let promise = cache.get(propertyKey);
      if (promise) return promise;

      promise = originFunction.apply(this, args);

      cache.set(propertyKey, promise);
      promise.finally(() => cache.delete(propertyKey));
      return promise;
    };

    return descriptor;
  };
}

export default debouncePromise;
