const rndRange = (start: number, end?: number) => {
  if (typeof end === "undefined") {
    end = start;
    start = 0;
  }
  return start + Math.floor(Math.random() * (end - start + 1));
};

const symbols = [".", "_", "-", "~", "°", "+"];
const rndSymbol = () => symbols[rndRange(symbols.length - 1)];

class Galaxy {
  static EmptySymbol = " ";
  width: number;
  height: number;
  field!: string[][];
  constructor(width: number, height: number) {
    this.width = width;
    this.height = height;
    this.reset();
  }
  reset() {
    this.field = Array(this.height)
      .fill(null)
      .map(() => Galaxy.EmptySymbol.repeat(this.width).split(""));
  }
  set(x: number, y: number, symbol: string) {
    if (x < 0 || x > this.width - 1 || y < 0 || y > this.height - 1) return;
    this.field[y][x] = symbol;
  }
  buildTrail() {
    let y = rndRange(this.height - 1);
    let x = rndRange(this.width - 1);
    let len = rndRange(1, this.width - x);

    while (len--) {
      x += 1;
      y += rndRange(-1, 1);
      this.set(x, y, rndSymbol());
    }
  }
  buildStar() {
    const ys = rndRange(this.height - 1);
    const xs = rndRange(this.width - 1);

    switch (rndRange(5)) {
      case 0:
        //  |
        // -o-
        //  |
        this.set(xs, ys, "o");
        this.set(xs, ys + 1, "|");
        this.set(xs, ys - 1, "|");
        this.set(xs + 1, ys, "-");
        this.set(xs - 1, ys, "-");
        break;
      case 1:
        this.set(xs, ys, ".");
        break;
      case 2:
        this.set(xs, ys, "`");
        break;
      case 3:
        this.set(xs, ys, "~");
        this.set(xs + 1, ys, "°");
        break;
      case 4:
        this.set(xs, ys, ",");
        break;
      case 5:
        this.set(xs, ys, "+");
        break;
      default:
        this.set(xs, ys, "*");
        break;
    }
  }
  toString(message = "—— Yumemi") {
    return (
      this.field.map((row) => row.join("")).join("\n") +
      "\n" +
      message.padStart(this.width, Galaxy.EmptySymbol)
    );
  }
}

export function genSecretGalaxy(): string {
  const galaxy = new Galaxy(79, 40);

  let trails = rndRange(17, 23);
  let stars = rndRange(35, 45);
  while (trails--) {
    galaxy.buildTrail();
  }
  while (stars--) {
    galaxy.buildStar();
  }
  const message =
    "This galaxy is generate for hide secrets in terminal. [Yumemi]";
  return (
    "\n" +
    [
      ...galaxy.field.map((row) => row.join("")),
      message.padStart(galaxy.width, Galaxy.EmptySymbol),
    ]
      .map((i) => `#${i}`)
      .join("\n") +
    "\n"
  );
}
