import path from "path";

import findUp from "find-up";

export async function moduleDir(): Promise<string> {
  const packageConfig = await findUp("package.json", {
    cwd: (require.main && require.main.filename) || __dirname,
  });
  if (!packageConfig) {
    console.error("can't find current package");
    throw process.exit(1);
  }
  return path.dirname(packageConfig);
}

export async function moduleResolve(subPath: string): Promise<string> {
  return path.resolve(await moduleDir(), subPath);
}

export async function projectDir(): Promise<string> {
  let probePath: string | undefined = "";
  probePath =
    (await findUp(".git", { type: "directory" })) ||
    (await findUp("package.json"));
  if (!probePath) {
    console.error("Project not found");
    throw process.exit(1);
  }
  return path.dirname(probePath);
}
