import os from "os";
import path from "path";

import fs from "mz/fs";

export const ConfigDir = path.resolve(os.homedir(), ".yumemi");

const JsonDBFileMap = new WeakMap<typeof JsonDB, string>();

export function JsonDBFile(file: string) {
  return function (target: typeof JsonDB): void {
    JsonDBFileMap.set(target, path.resolve(ConfigDir, file));
  };
}

export default abstract class JsonDB {
  ensureConfigDir(): void {
    try {
      fs.statSync(ConfigDir);
    } catch (e) {
      fs.mkdirSync(ConfigDir);
    }
    try {
      fs.statSync(this.filepath);
    } catch (e) {
      fs.writeFileSync(this.filepath, "{}", { encoding: "utf8" });
    }
    fs.chmodSync(this.filepath, 0o600);
  }
  get filepath(): string {
    return JsonDBFileMap.get(this.constructor as typeof JsonDB) as string;
  }
  load(): void {
    try {
      Object.assign(
        this,
        JSON.parse(fs.readFileSync(this.filepath, { encoding: "utf8" }))
      );
      // eslint-disable-next-line no-empty
    } catch (e) {}
  }
  save(): void {
    this.ensureConfigDir();
    fs.writeFileSync(this.filepath, JSON.stringify(this), {
      encoding: "utf8",
    });
  }
}
