enum Box {
  Node = "├─ ",
  LastNode = "└─ ",
  LinkedIndent = "|  ",
  Indent = "   ",
}

interface TreeViewRenderOptions {
  depth: number;
}

interface TreeViewLine<T> {
  boxes: string[];
  view: string;
  node: T;
}

export default abstract class TreeView<T extends { children?: T[] }> {
  root: T;

  constructor(root: T) {
    this.root = root;
  }
  abstract renderNode(node: T): string;

  render({ depth }: TreeViewRenderOptions): TreeViewLine<T>[] {
    const lines: TreeViewLine<T>[] = [];

    const renderTree = (node: T, depth: number, boxes: Box[] = []): void => {
      lines.push({
        boxes: boxes.slice(),
        view: this.renderNode(node),
        node: node,
      });

      if (depth <= 0) return;

      if (!node.children || node.children.length === 0) return;

      const subBoxes = boxes.slice();
      const lastBoxIndex = boxes.length - 1;
      const lastBox = boxes[lastBoxIndex];

      if (lastBox === Box.Node) {
        subBoxes[lastBoxIndex] = Box.LinkedIndent;
      } else if (lastBox === Box.LastNode) {
        subBoxes[lastBoxIndex] = Box.Indent;
      }

      node.children.forEach((child, index, { length }) => {
        const isLastOne = index === length - 1;
        renderTree(child, depth - 1, [
          ...subBoxes,
          isLastOne ? Box.LastNode : Box.Node,
        ]);
      });
    };

    renderTree(this.root, depth);
    return lines;
  }
}
