import axios from "axios";
import type {
  AxiosDefaults,
  AxiosInterceptorManager,
  AxiosPromise,
  AxiosRequestConfig,
  AxiosResponse,
} from "axios";

import tokenCache from "./token-cache";

export interface GetReqConfig<T> extends AxiosRequestConfig {
  params: T;
}

export interface PostReqConfig<T> extends AxiosRequestConfig {
  data: T;
}

export interface VaultAxiosInstance {
  (config: AxiosRequestConfig): AxiosPromise;
  (url: string, config?: AxiosRequestConfig): AxiosPromise;
  defaults: AxiosDefaults;
  interceptors: {
    request: AxiosInterceptorManager<AxiosRequestConfig>;
    response: AxiosInterceptorManager<AxiosResponse>;
  };
  getUri(config?: AxiosRequestConfig): string;
  request<Req = unknown, Resp = unknown>(
    config: GetReqConfig<Req> | PostReqConfig<Req>
  ): Promise<Resp>;
  get<Resp = unknown>(
    url: string,
    config?: GetReqConfig<unknown>
  ): Promise<Resp>;
  get<Req = unknown, Resp = unknown>(
    url: string,
    config?: GetReqConfig<Req>
  ): Promise<Resp>;
  delete<Req = unknown, Resp = unknown>(
    url: string,
    config?: GetReqConfig<Req>
  ): Promise<Resp>;
  head<Req = unknown, Resp = unknown>(
    url: string,
    config?: GetReqConfig<Req>
  ): Promise<Resp>;
  options<Req = unknown, Resp = unknown>(
    url: string,
    config?: GetReqConfig<Req>
  ): Promise<Resp>;
  post<Req = unknown, Resp = unknown>(
    url: string,
    data?: Req,
    config?: AxiosRequestConfig
  ): Promise<Resp>;
  put<Req = unknown, Resp = unknown>(
    url: string,
    data?: Req,
    config?: AxiosRequestConfig
  ): Promise<Resp>;
  patch<Req = unknown, Resp = unknown>(
    url: string,
    data?: Req,
    config?: AxiosRequestConfig
  ): Promise<Resp>;
}

export interface KvResponse<T> {
  request_id: string;
  lease_id: string;
  renewable: boolean;
  lease_duration: number;
  data: T;
}

export const baseURL = "https://vault.geektr.co/v1";

const vaultAxios: VaultAxiosInstance = axios.create({ baseURL });

vaultAxios.interceptors.request.use(async (value) => {
  await tokenCache.initialize();
  value.headers = value.headers || {};
  value.headers["X-Vault-Token"] = tokenCache.token;
  return value;
});

vaultAxios.interceptors.response.use((value) => {
  return value.data;
});

export default vaultAxios;
